@@ -7,6 +7,7 @@ import re
 import copy
 import itertools
 import numbers
+import pkg_resources
 
 
 # Python3 support
@@ -22,8 +23,7 @@ else:
         # Support for GAE runner
         from itertools import imap as map
     import urlparse
-    str_types = (str, unicode)
-
+    str_types = (basestring,)
 
 """
 Trafaret is tiny library for data validation
@@ -33,7 +33,11 @@ Look at doctests for usage examples
 
 __all__ = ("DataError", "Trafaret", "Any", "Int", "String",
            "List", "Dict", "Or", "Null", "Float", "Enum", "Callable",
-           "Call", "Forward", "Bool", "Type", "Mapping", "guard", "Key")
+           "Call", "Forward", "Bool", "Type", "Mapping", "guard", "Key",
+           "Tuple", "Atom", "Email", "URL")
+
+ENTRY_POINT = 'trafaret'
+_empty = object()
 
 
 def py3metafix(cls):
@@ -498,17 +502,37 @@ class String(Trafaret):
     'wqerwqer'
     >>> extract_error(String(regex='^\w+$'), 'wqe rwqer')
     'value does not match pattern'
+    >>> String(min_length=2, max_length=3).check('123')
+    '123
+    >>> extract_error(String(min_length=2, max_length=6), '1')'
+    'String is shorter than 2 characters'
+    >>> extract_error(String(min_length=2, max_length=6), '1234567')
+    'String is longer than 6 characters'
+    >>> String(min_length=2, max_length=6, allow_blank=True)
+    Traceback (most recent call last):
+    ...
+    AssertionError: Either allow_blank or min_length should be specified, not both
+    >>> String(min_length=0, max_length=6, allow_blank=True).check('123')
+    '123'
     """
 
-    def __init__(self, allow_blank=False, regex=None):
+    def __init__(self, allow_blank=False, regex=None, min_length=None, max_length=None):
+        assert not (allow_blank and min_length), \
+            "Either allow_blank or min_length should be specified, not both"
         self.allow_blank = allow_blank
         self.regex = re.compile(regex) if isinstance(regex, str_types) else regex
+        self.min_length = min_length
+        self.max_length = max_length
 
     def check_and_return(self, value):
         if not isinstance(value, str_types):
             self._failure("value is not a string")
         if not self.allow_blank and len(value) is 0:
             self._failure("blank value is not allowed")
+        if self.min_length is not None and len(value) < self.min_length:
+            self._failure('String is shorter than %s characters' % self.min_length)
+        if self.max_length is not None and len(value) > self.max_length:
+            self._failure('String is longer than %s characters' % self.max_length)
         if self.regex is not None:
             match = self.regex.match(value)
             if not match:
@@ -539,6 +563,8 @@ class Email(String):
         r'|^"([\001-\010\013\014\016-\037!#-\[\]-\177]|\\[\001-011\013\014\016-\177])*"' # quoted-string
         r')@(?P<domain>(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+[A-Z]{2,6}\.?$)'  # domain
         r'|\[(25[0-5]|2[0-4]\d|[0-1]?\d?\d)(\.(25[0-5]|2[0-4]\d|[0-1]?\d?\d)){3}\]$', re.IGNORECASE)  # literal form, ipv4 address (SMTP 4.1.3)
+    min_length = None
+    max_length = None
 
     def __init__(self, allow_blank=False):
         self.allow_blank = allow_blank
@@ -578,6 +604,8 @@ class URL(String):
         r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})' # ...or ip
         r'(?::\d+)?' # optional port
         r'(?:/?|[/?]\S+)$', re.IGNORECASE)
+    min_length = None
+    max_length = None
 
     def __init__(self, allow_blank=False):
         self.allow_blank = allow_blank
@@ -755,9 +783,26 @@ class Key(object):
 
     """
     Helper class for Dict.
+    
+    >>> default = lambda: 1
+    >>> Key(name='test', default=default)
+    <Key "test">
+    >>> Key(name='test', default=default).pop({}).__next__()
+    ('test', 1)
+    >>> Key(name='test', default=2).pop({}).__next__()
+    ('test', 2)
+    >>> default = lambda: None
+    >>> Key(name='test', default=default).pop({}).__next__()
+    ('test', None)
+    >>> Key(name='test', default=None).pop({}).__next__()
+    ('test', None)
+    >>> Key(name='test').pop({}).__next__()
+    ('test', DataError(is required))
+    >>> list(Key(name='test', optional=True).pop({}))
+    []
     """
 
-    def __init__(self, name, default=None, optional=False, to_name=None, trafaret=None):
+    def __init__(self, name, default=_empty, optional=False, to_name=None, trafaret=None):
         self.name = name
         self.to_name = to_name
         self.default = default
@@ -765,9 +810,14 @@ class Key(object):
         self.trafaret = trafaret or Any()
 
     def pop(self, data):
-        if self.name in data or self.default is not None:
+        if self.name in data or self.default is not _empty:
+            if callable(self.default):
+                default = self.default()
+            else:
+                default = self.default
+            # default = callable(self.default) and self.default() or self.default
             yield self.get_name(), catch_error(self.trafaret,
-                    data.pop(self.name, self.default))
+                    data.pop(self.name, default))
             raise StopIteration
         if not self.optional:
             yield self.name, DataError(error='is required')
@@ -1055,6 +1105,11 @@ class Forward(Trafaret):
                         {"name": "bar", "children": []} \
                      ]}) == {'children': [{'children': [], 'name': 'bar'}], 'name': 'foo'}
     True
+    >>> empty_node = Forward()
+    >>> empty_node
+    <Forward(None)>
+    >>> extract_error(empty_node, 'something')
+    'trafaret not set yet'
     """
 
     def __init__(self):
@@ -1070,6 +1125,8 @@ class Forward(Trafaret):
         self.trafaret = self._trafaret(trafaret)
 
     def check_and_return(self, value):
+        if self.trafaret is None:
+            self._failure('trafaret not set yet')
         return self.trafaret.check(value)
 
     def __repr__(self):
@@ -1133,8 +1190,10 @@ def guard(trafaret=None, **kwargs):
                            " trafaret or kwargs")
     if not trafaret:
         trafaret = Dict(**kwargs)
+
     def wrapper(fn):
         argspec = inspect.getargspec(fn)
+
         @functools.wraps(fn)
         def decor(*args, **kwargs):
             fnargs = argspec.args
@@ -1161,7 +1220,6 @@ def guard(trafaret=None, **kwargs):
 
 
 def ignore(val):
-
     """
     Stub to ignore value from trafaret
     Use it like:
@@ -1169,7 +1227,6 @@ def ignore(val):
     >>> a = Int >> ignore
     >>> a.check(7)
     """
-
     pass
 
 
@@ -1187,6 +1244,7 @@ def catch_error(checker, *a, **kw):
     except DataError as error:
         return error
 
+
 def extract_error(checker, *a, **kw):
 
     """
@@ -1197,3 +1255,15 @@ def extract_error(checker, *a, **kw):
     if isinstance(res, DataError):
         return res.as_dict()
     return res
+
+
+def load_contrib():
+    for entrypoint in pkg_resources.iter_entry_points(ENTRY_POINT):
+        try:
+            trafaret_class = entrypoint.load()
+            setattr(sys.modules[__name__], trafaret_class.__name__,
+                    trafaret_class)
+        except pkg_resources.DistributionNotFound as err:
+            # TODO: find a way to pass error message upper
+            pass
+load_contrib()
